""" All the errors that this package can rise are defined in this file.

    Copyright 2008 Wireless Cables Inc. <www.aircable.net>
    Copyright 2008 Naranjo, Manuel Francisco <manuel@aircable.net>

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
class SPPException(Exception):
    pass

class SPPNotConnectedException(SPPException):
    pass
    
class SPPNotImplemented(SPPException):
    pass



#   OpenProximity2.0 is a proximity marketing OpenSource system.        
#   Copyright (C) 2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#                                                                          
#   This program is free software; you can redistribute it and/or modify  
#   it under the terms of the GNU General Public License as published by  
#   the Free Software Foundation version 2 of the License.                
#                                                                          
#   This program is distributed in the hope that it will be useful,       
#   but WITHOUT ANY WARRANTY; without even the implied warranty of        
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         
#   GNU General Public License for more details.                          
#                                                                          
#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc.,
#   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.   

'''
A few useful constants.

Note: We should move return codes here!
'''

BLUEZ='org.bluez'
BLUEZ_PATH='/'
BLUEZ_ADAPTER='org.bluez.Adapter'
BLUEZ_MANAGER='org.bluez.Manager'

AIRCABLE_MAC=['00:50:C2', '00:25:BF']

UUID={
    'spp': '00001101-0000-1000-8000-00805f9b34fb',
    'opp': '00001105-0000-1000-8000-00805f9b34fb',
    'ftp': '00001106-0000-1000-8000-00805f9b34fb',
}




# -*- coding: utf-8 -*-
#    OpenProximity2.0 is a proximity marketing OpenSource system.
#    Copyright (C) 2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
import const
import logging
import logging.handlers
import os, sys, time

logging.basicConfig()

def isAIRcable(address):
    return address[:8].upper() in const.AIRCABLE_MAC

from openproximity.lib import preset as settings

clogger = logging.StreamHandler()
clogger.setFormatter(logging.Formatter(settings.DEBUG_CONSOLE_FORMAT))
clogger.setLevel(getattr(logging, settings.DEBUG_CONSOLE, logging.NOTSET))

print logging.getLevelName(clogger.level)

global LOG_FLAG
LOG_FLAG=False

flogger = None
flogger_file=None
flogger_file = os.path.join(settings.DEBUG_PATH, settings.DEBUG_FILENAME)

try:
    open(flogger_file, "w")
    flogger=logging.handlers.RotatingFileHandler(flogger_file, 
        maxBytes=1024*settings.DEBUG_MAXSIZE,
        backupCount=settings.DEBUG_COUNT
    )
    format=logging.Formatter(settings.DEBUG_FORMAT)
    flogger.setLevel(getattr(logging, settings.DEBUG_LEVEL, logging.NOTSET))
    flogger.setFormatter(format)
except:
    LOG_FLAG = True
    flogger = logging.NullHandler()

loggers={}

def getLogger(name=None):
    global LOG_FLAG
    if name in loggers:
        return loggers[name]

    logger = logging.getLogger(name)
    logger.setLevel(logging.NOTSET)

    if name and name in settings.DEBUG_DISABLES:
        logging.getLogger(None).info("disabled")
        return logger

    #print name, clogger, clogger.level
    logger.addHandler(clogger)

    if LOG_FLAG:
        LOG_FLAG = False
        logger.warning("Can't write to log file at %s" % flogger_file)
    logger.addHandler(flogger)

    loggers[name]=logger
    logging.getLogger(None).info("Added logger for %s" % name)
    return logger

getLogger(None).setLevel(logging.NOTSET)

def trace():
    try:
        #from pydbgr.api import debug as set_trace
        from pudb import set_trace
        import urwid
        def t(*args, **kwargs):
            pass
        urwid.raw_display.Screen.signal_init=t
        urwid.raw_display.Screen.signal_restore=t
    except Exception, err:
        logger.warning("Using non multithreaded pdb")
        from pdb import set_trace 
    return set_trace()

def register_debug_shell():
    ''' 
        a helper function that will allow to start pdb or pudb
        when CTRL-C is received
    '''
    if 'PDB' not in os.environ:
        return

    logger.debug("Registering PDB debugger")
    def signal_handler(signal, frame):
        trace()
    import signal
    signal.signal(signal.SIGINT, signal_handler)

register_debug_shell()

def get_subclass(object):
    for related in object._meta.get_all_related_objects():
        if type(object) in related.model._meta.get_parent_list():
            if hasattr(object,related.var_name):
                return get_subclass(getattr(object, related.var_name))
    return object

def get_subclasses(klass):
    out = [klass, ]
    if len(klass.__subclasses__()) > 0:
        for k in klass.__subclasses__():
            out.extend(get_subclasses(k))
    return out



"""
    A simple class for connecting to spp servers.

    Copyright 2008 Wireless Cables Inc. <www.aircable.net>
    Copyright 2008 Naranjo, Manuel Francisco <manuel@aircable.net>

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

import socket
import dbus

from errors import *
from sppBase import *
from xml.dom.minidom import parseString
import net.aircable.sdp as sdp
import net.aircable.const as const

from net.aircable.utils import getLogger
logger=getLogger(__name__)

class sppClient(sppBase):
	target = None
	
	def __init__( self, target, 
                        channel = -1,
                        service = 'spp',
                        device  = None  ):
	    sppBase.__init__(self, 
			    channel = channel, 
			    service = service, 
			    device  = device);
	    logger.info("sppClient.__init__")
	    logger.info("target: %s" % target )
	    self.target = target


	def __browsexml(self, doc):
	    record=doc.documentElement
	    
	    for node in record.childNodes:
	        if node.nodeType == node.ELEMENT_NODE and node.getAttribute('id') == '0x0004':
			val = node.getElementsByTagName('uint8')
			return int(val[0].getAttribute('value'), 16)
				    
	def resolveService(self, target, service='spp' ):
	    logger.info("Resolving service %s for device %s...."
		    %(service, target) 
		);
	    
	    adapter = dbus.Interface( self.getAdapterObjectPath(), 
						    'org.bluez.Adapter' )
	    
	    return sdp.resolve(target, 
		uuid=const.UUID.get('spp', '00001101-0000-1000-8000-00805f9b34fb'),
		adapter=adapter, bus=self.bus)

	def connect(self):
	    '''
		Starts the connection
	    '''
	    logger.info("connecting")
	    if not self.target:
		raise SPPNotImplemented, 'Scanning is not supported, you need to do that by your own'
	    
	    if self.channel < 1:
		self.channel = self.resolveService( self.target, self.service)
		    
	    if (self.socket == None):
		logger.debug('creating socket')
		self.socket = socket.socket( getattr(socket, 'AF_BLUETOOTH', 31),
						socket.SOCK_STREAM, 
						getattr(socket, 'BTPROTO_RFCOMM', 3) );
	    #Let BlueZ decide outgoing port
	    logger.debug('binding to %s, %i' % ( self.device , 0 ))
	    self.socket.bind( (self.device,0) );
		
	    logger.debug('connecting to %s, %i' % ( self.target, self.channel ))
	    self.socket.connect( (self.target, self.channel) );


if __name__ == '__main__':
    import sys
    
    if ( len(sys.argv) < 2 ):
	print "Usage %s <target> [ <channel>  <adapter> ]" % (sys.argv[0])
	sys.exit(1)
    
    if ( len(sys.argv) > 2 ):
	chan = sys.argv[2]
    else:
	chan = -1
    
    if ( len(sys.argv) > 3 ):
	dev = sys.argv[3]
    else:
	dev = None
	
    a = sppClient(
	    target	=sys.argv[1],
	    channel	=chan,
	    service	='spp',
	    device	=dev
	);
    
    a.connect()
    
    a.sendLine('hi');
    print a.readLine();
    a.sendLine('bye');
                
    a.disconnect();
                    



# Modified by Manuel Naranjo for OpenProximity2 <manuel@aircable.net> 2009
# This autoreloader will reload on different events:
#   * RELOAD variable is TRUE (allowing incode reload)
#   * code changed
#   * SIGUSR1 is recieved
# Autoreloader will kill childs when parent dies
# sigusr1 to parent triggers parent killing
#
# Autoreloading launcher.
# Borrowed from Peter Hunt and the CherryPy project (http://www.cherrypy.org).
# Some taken from Ian Bicking's Paste (http://pythonpaste.org/).
#
# Portions copyright (c) 2004, CherryPy Team (team@cherrypy.org)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright notice,
#       this list of conditions and the following disclaimer in the documentation
#       and/or other materials provided with the distribution.
#     * Neither the name of the CherryPy Team nor the names of its contributors
#       may be used to endorse or promote products derived from this software
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os, sys, time, signal, threading
from net.aircable.utils import getLogger
logger=getLogger(__name__)

__all__=['isParent', 'main', 'RESTART']

try:
    import thread
except ImportError:
    import dummy_thread as thread

# This import does nothing, but it's necessary to avoid some race conditions
# in the threading module. See http://code.djangoproject.com/ticket/2330 .
try:
    import threading
except ImportError:
    pass


RUN_RELOADER = True
RESTART = False

_mtimes = {}
_win = (sys.platform == "win32")

def isParent():
    '''
    This method will let the running process if it's the one that forked, or the
    forked one.
    '''
    if os.environ.get('DEBUG', None) is None:
	return os.getpid() == int(os.environ.get('RUN_PARENT', os.getpid()))
    else:
	return True

def handle_child(signum, frame):
    '''
    Gets called when a signal is received, by default we kill the child. The 
    parent will detect this and start a new child (when the return value was 3)
    '''
    sys.exit(3)
    
def handle_parent(signum, frame):
    '''
    When the parent gets the kill signal it will stop it self no forking more
    childrens.
    '''
    logger.info("Parent killed, doing exit")
    sys.exit(0)

def code_changed():
    '''
    An utility method that will check if there are any code changes in the inuse
    modules. When there's a change this method will return TRUE. Based on django
    code.
    '''
    global _mtimes, _win
    for filename in filter(lambda v: v, map(lambda m: getattr(m, "__file__", None), sys.modules.values())):
        if filename.endswith(".pyc") or filename.endswith(".pyo"):
            filename = filename[:-1]
        if not os.path.exists(filename):
            continue # File might be in an egg, so it can't be reloaded.
        stat = os.stat(filename)
        mtime = stat.st_mtime
        if _win:
            mtime -= stat.st_ctime
        if filename not in _mtimes:
            _mtimes[filename] = mtime
            continue
        if mtime != _mtimes[filename]:
            _mtimes = {}
            return True
    return False

def isRunning(pid):
    '''
    This method will inquiry PID to know if it's running.
    '''
    try:
	os.kill(pid, 0)
	return True
    except OSError, err:
	return False

def reloader_thread(t):
    '''
    This thread runs in the child. It will check if the parent is running, if
    so it will continue. If there's a change in the code, or if the worker 
    thread ended it will kill it self with signal 3.
    If no parent is available it will kill it self with error 0 (no parent, no
    way to restart)
    '''
    parent_pid=int(os.environ.get('RUN_PARENT'))
    while RUN_RELOADER:
	
	# check if parent is running
	if not isRunning(parent_pid):
	    # no parent, time to end
	    sys.exit(0)
	
        if code_changed() or RESTART or not t.isAlive():
            sys.exit(3) # force reload
        time.sleep(1)

def restart_with_reloader():
    '''
    This method will spawn a new child when the exit code is 3.
    '''
    while True:
        args = [sys.executable] + sys.argv
        if sys.platform == "win32":
            args = ['"%s"' % arg for arg in args]
        new_environ = os.environ.copy()
        new_environ["RUN_MAIN"] = 'true'
        exit_code = os.spawnve(os.P_WAIT, sys.executable, args, new_environ)
        if exit_code != 3:
            break;
    sys.exit(exit_code)

def python_reloader(main_func, args, kwargs):
    '''
    Internal method that will setup the forking process. If it's the parent
    then it will do the forks, otherwise it will start a new worker thread and
    setup a reloader thread.
    '''
    if os.environ.get("RUN_MAIN") == "true":
	signal.signal(signal.SIGUSR1, handle_child)
	t = threading.Thread(target=main_func, args=args, kwargs=kwargs)
	t.start()
        try:
            reloader_thread(t)
        except KeyboardInterrupt:
            pass
	except Exception, err:
	    logger.exception(err)
    else:
	# this is the parent pid
        try:
	    signal.signal(signal.SIGUSR1, handle_parent)
	    os.environ['RUN_PARENT']=str(os.getpid())
	    restart_with_reloader()
        except KeyboardInterrupt:
            pass

def main(main_func, args=(), kwargs={}):
    '''
    The only method you want to use from the outside, it will decide to use 
    either a forking schema when DEBUG is not defined. Or it will just call your
    method and don't fork at all, when you want to DEBUG.
    '''
    if os.environ.get('DEBUG', None) is None:
	python_reloader(main_func, args, kwargs)
    else:
	main_func(*args, **kwargs)

if __name__ == '__main__':
    def test():
	pass
    
    if isParent():
	print "I'm the parent:", os.getpid()
    else:
	print "I'm a child:", os.getpid()
    main(test)

    


#    OpenProximity2.0 is a proximity marketing OpenSource system.
#    Copyright (C) 2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#!/usr/bin/python

import sys
import dbus
import re

from utils import getLogger
logger=getLogger(__name__)

from lxml import etree

CHANNEL_XPATH=etree.XPath("/record/attribute[@id='0x0004']/sequence/sequence/uuid[@value='0x0003']/../uint8/@value")

OBEX_UUID='00001105-0000-1000-8000-00805f9b34fb'
FTP_UUID='00001106-0000-1000-8000-00805f9b34fb'

							
def resolve(address, uuid, adapter=None, bus=dbus.SystemBus()):
	try:
	    adapter.CreateDevice(address)
	    logger.debug("Device created: %s" % address)
	except:
	    logger.debug("Device all ready known: %s" % address)
	    
	path = adapter.FindDevice(address)
	
	device = dbus.Interface(bus.get_object("org.bluez", path),
							"org.bluez.Device")
	properties = device.GetProperties()
	
	if uuid in properties['UUIDs']:
	    services = device.DiscoverServices(uuid);
		
	    for key in services.keys():
		root=etree.XML(str(services[key]))
		res=CHANNEL_XPATH(root)
		if len(res)>0:
		    return int(res[0], 16)
	
	# lets check if the service is there now
	#adapter.RemoveDevice(path)
	
	raise Exception("UUID not found")
		
if __name__ == '__main__':
    bus = dbus.SystemBus()
    manager = dbus.Interface(bus.get_object("org.bluez", "/"), "org.bluez.Manager")

    adapter = dbus.Interface(bus.get_object("org.bluez", manager.DefaultAdapter()),
							"org.bluez.Adapter")

    print resolve('00:15:E9:F5:BA:FE', OBEX_UUID, adapter, bus)
    print resolve('00:15:E9:F5:BA:FE', FTP_UUID, adapter, bus)



#    OpenProximity2.0 is a proximity marketing OpenSource system.
#    Copyright (C) 2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
import dbus
import const
import utils
from net.aircable.utils import getLogger
logger=getLogger(__name__)

class Adapter(object):
    '''A wrapper arround an adapter'''
    dbus_path = None
    dbus_interface = None
    bt_address = None
    is_aircable = False
    bus = None

    def __init__(self, bus, path, name="OpenProximity 2.0"):
        self.dbus_path = path
        self.dbus_interface = dbus.Interface( bus.get_object(const.BLUEZ, path), const.BLUEZ_ADAPTER )
        #self.bt_address = self.dbus_interface.GetAddress()
        self.bt_address=str(self.dbus_interface.GetProperties()['Address'])
        self.dbus_interface.SetProperty('Powered', True)
        self.is_aircable = utils.isAIRcable(self.bt_address)
        self.bus = bus
        self.dbus_interface.SetProperty('Name', name)
        logger.debug("Initializated Adapter(%s, %s)" % (self, self.is_aircable)) 

    def __str__(self):
        return '%s, %s' % (self.bt_address, self.dbus_path)

if __name__=='__main__':
    import dbus.glib
    import gobject
    adapter = Adapter( dbus.SystemBus(),'/org/bluez/hci0')
    print adapter, adapter.aircable



#    OpenProximity2.0 is a proximity marketing OpenSource system.
#    Copyright (C) 2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

'''
Scanners signals
'''

BASE_SIGNAL = 100
MAX_SIGNAL = BASE_SIGNAL + 200

DONGLES_ADDED=BASE_SIGNAL 
NO_DONGLES=BASE_SIGNAL+1

CYCLE_COMPLETE=BASE_SIGNAL+20
CYCLE_START=BASE_SIGNAL+21
CYCLE_SCAN_DONGLE=BASE_SIGNAL+22
CYCLE_SCAN_DONGLE_COMPLETED=BASE_SIGNAL+23
DONGLE_NOT_AVAILABLE=BASE_SIGNAL+24

FOUND_DEVICE=BASE_SIGNAL+30

TEXT = {
	DONGLES_ADDED:			'Dongles added',
	NO_DONGLES: 			'No dongles to add',
	
	CYCLE_COMPLETE: 		'Scan cycle completed',
	CYCLE_START:			'Scan cycle start',
	CYCLE_SCAN_DONGLE:		'Scan cycle on dongle',
	CYCLE_SCAN_DONGLE_COMPLETED:	'Scan cycle on dongle completed',
	DONGLE_NOT_AVAILABLE:		'Scan dongle not available',
	
	FOUND_DEVICE:			'Found device',
}

POST = {
	DONGLES_ADDED:			'scanner_dongle_added',
	NO_DONGLES: 			'scanner_no_dongle',
	
	CYCLE_COMPLETE: 		'scanner_cycle_complete',
	CYCLE_START:			'scanner_cycle_start',
	CYCLE_SCAN_DONGLE:		'scanner_cycle_scan_start',
	CYCLE_SCAN_DONGLE_COMPLETED: 	'scanner_cycle_scan_end',
	DONGLE_NOT_AVAILABLE:		'scanner_cycle_dongle_not_available',
	
	FOUND_DEVICE:			'scanner_cycle_found_device',
}



"""
    Base class for both sppClient and sppServer, rfcomm clients/servers
    wrappers.

    Copyright 2008 Wireless Cables Inc. <www.aircable.net>
    Copyright 2008 Naranjo, Manuel Francisco <manuel@aircable.net>

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

import socket, time

import dbus

from re import compile
from errors import *
from select import select

try:
    from net.aircable.utils import getLogger
    logger = getLogger(__name__)
except:
    import logging
    logger = logging.getLogger('sppAIRcable')

    console = logging.StreamHandler()
    formatter = logging.Formatter('%(name)-12s: %(levelname)-8s %(message)s')
    console.setFormatter(formatter)
    logger.addHandler(console)
    logger.setLevel(logging.DEBUG)


class sppBase:
	'''
	    Base class for rfcomm wrappers regardless of it\'s master or slave
	    behaviour.
	'''
        socket  = None;
        channel = None;
	service = None;
	device  = None;
	
	bus 	  = dbus.SystemBus();
	__pattern = compile(r'.*\n');
	
	new_bluez_api = True;
	
	def getDefaultDeviceAddress(self):
	    obj     = self.bus.get_object( 'org.bluez', '/' )
	    manager = dbus.Interface( obj, 'org.bluez.Manager' )
	    obj     = self.bus.get_object( 'org.bluez', 
						    manager.DefaultAdapter() )
	    adapter = dbus.Interface( obj, 'org.bluez.Adapter' )
	    address = adapter.GetProperties()['Address']
	    return address
	    
	def getAdapterObjectPath(self):
            bluez_path   = self.bus.get_object( 'org.bluez', '/' )
            manager = dbus.Interface( bluez_path, 'org.bluez.Manager' )

            return self.bus.get_object( 'org.bluez',
                            manager.FindAdapter(self.device)
                        )

	def __init__(self, socket):
	    '''
		    You use this constructor when you all ready have a socket
	        
		arguments:
		    socket descriptor
	    '''
	    self.__socket = socket;

	def __init__( self,
			channel = -1, 
			service = 'spp',
			device  = None  ):
	    '''
		More general constructor. You will use this one when you want
		sppClient to make the conneciton.
	    
		arguments:
		    channel: Channel to be used for establishing the connection.
		    service: Service to use when you want sppClient to do service
			     Discovery.
		    device:  Bluetooth Address of the local device you want to 
			     use for making the connection, None for default.
	    '''

	    self.channel = int(channel);
	    self.service = service;
	    
	    if not device:
		device = self.getDefaultDeviceAddress()
	    
	    self.device  = device;
	    
	    self.iface = dbus.Interface( 
		    self.getAdapterObjectPath(),
		    'org.bluez.Adapter')
            
	    logger.info("sppBase.__init__");
	    logger.info("Channel: %s" % channel );
	    logger.info("Service: %s" % service );
	    logger.info("Device: %s"  % device  );

	def checkConnected(self, message =''):
		if self.socket == None:
		    raise SPPNotConnectedException, message
		    
	def disconnect(self, force=False):
	    if not force:
		self.checkConnected("Can't close if it's opened");
	    logger.info("Closing socket");
	    self.socket.shutdown(socket.SHUT_RDWR);
	    self.socket.close()
	    self.socket = None

	def send(self, text):
	    '''
		Send something to the port.
	    
		Arguments: what to send
	    '''
	    self.checkConnected('Can\'t send if not connected');
	    
	    logger.debug(">> %s" % text)
		
	    ret = self.socket.sendall(text, socket.MSG_WAITALL);
	    
	    if ret and int(ret) != text.length():
		raise SPPException, "Failed to send all data"
	
	def sendLine(self, text):
	    """
		Send a line instead of only text, this will add \n
	    """
	    self.send("%s\n\r" % text)

	def read(self, bytes=10, log=True, timeout=None):
	    '''
		Read binary data from the port.
	    
		Arguments:
		    bytes: Amount of bytes to read
	    '''
	    self.checkConnected('Can\'t read if not connected');
	    
	    ret = None
	    
	    if timeout is None:
		timeout = self.socket.gettimeout()
	    
	    if timeout is not None:
		start = time.time()
	    
	    while [ 1 ]:
		sel = select( (self.socket,), (), (), 0)
		if len(sel[0]) > 0:
		    ret = self.socket.recv(bytes);
		    break
		if timeout is not None and time.time() - start > timeout:
		    raise SPPException("timeout reached on read")
		time.sleep(0.1)
	    
	    if( log ):
		logger.debug("<< %s" % ret)
	    
	    return ret

	def readLine(self):
	    '''
		Reads until \n is detected
	    '''
	    out = buffer("");
		
	    while ( 1 ):
		out += self.read(bytes=1, log=False);

		if self.__pattern.match(out):
			out = out.replace('\n', '');
			logger.debug("<< %s" % out )
			return out
	
	def readBuffer(self, honnor_eol=False,timeout=1, log=False):
	    '''
		Reads all the data inside the buffer
	    '''
	    out = buffer("");
#	    timeout__ = self.socket.gettimeout()
#	    self.socket.settimeout(timeout)
	    start = time.time()
	    try:
		while ( 1 ):
		    out += self.read(bytes=1, log=False, timeout=timeout)
		    if honnor_eol and self.__pattern.match(out):
			break;
		    if time.time() - start > timeout:
			break;
	    except SPPException, err:
		if log:
		    logger.exception(err)
	    
	    
	    return str(out).replace('\n', '')
#	    self.socket.settimeout(timeout__)
#	    return 
			
	#shell functions wrappers
	def shellGrabFile(self, file):
		out = "";
		self.sendLine("s%s" % file)
		
		while [ 1 ]:
			try:
				line = self.readLine()
			except socket.error:
				logger.info("Connection lost connection")
				break;

			logger.debug("line=%s" % line)
		
			if (line != None and line.find("DONE") >- 1):
				logger.debug("EOF")
				break;
			elif line.startswith(">") or len(line)>1 or line.find("GO")==-1:
				out += line
				out += "\n"
			#self.sendLine("GO")
		
		out=out.replace('>','')
		logger.debug("Got:\n%s" % out);
		return out
    
	def shellAskLine(self, number):
		'''Function wrapper for a shell operation'''
		logger.debug('shellAskLine %s' % number)
		#self.makeShellReady()
		self.sendLine("p%04i" % number)
		while [ 1 ]:
			line = self.readLine()
			if line.find("error")==-1:
				return line.replace('>', '').strip()
			else:
			    return ""

	def shellSetLine(self, number, content):
		'''Function wrapper for a shell operation'''
		self.__sendCommand("S%04i%s" % (number, content))
    
	def __sendCommand(self, command):
		logger.debug('__sendCommand %s' % command)
		#self.makeShellReady()
		self.sendLine(command)
		while [ 1 ]:
			line=self.readLine()
			if line.find('error')==-1:
				return line
    
	def shellExit(self):
		'''Function wrapper for a shell operation'''
		self.__sendCommand('e')

	def shellUpdate(self):
		'''Function wrapper for a shell operation'''
		self.__sendCommand('u')
    
	def shellPushIntoHistory(self, argument):
		'''Function wrapper for a shell operation'''
		self.__sendCommand("c%s" % argument)
	
	def shellDeleteFile(self, file):
		self.__sendCommand("d%s" % file)

	# stupid function that will try to sync both shells
	def makeShellReady(self):
		logger.debug("MakeShellReady()")
		line=""
		timeout=self.socket.gettimeout()
		self.socket.settimeout(1)

		while [ 1 ]:
			logger.debug('reading')
			try:
				line += self.read(200)
			except socket.error: # wait until socket timesout
				logger.debug('Shell cleaned')
				self.socket.settimeout(timeout)
				return line
				
	def endConnection(self, exit=True):
		self.shellPushIntoHistory(time.time())
		
		time.sleep(2)
		if exit:
			logger.info("Sending exit")
			self.shellExit()




#    OpenProximity2.0 is a proximity marketing OpenSource system.
#    Copyright (C) 2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

'''
Uploaders signals
'''

BASE_SIGNAL = 500
MAX_SIGNAL = BASE_SIGNAL + 200

DONGLES_ADDED=BASE_SIGNAL 
NO_DONGLES=BASE_SIGNAL+1

CYCLE_UPLOAD_DONGLE=BASE_SIGNAL+20

FILE_UPLOADED=BASE_SIGNAL+30
FILE_FAILED=BASE_SIGNAL+31

SDP_RESOLVED=BASE_SIGNAL+40
SDP_NORECORD=BASE_SIGNAL+41
SDP_TIMEOUT=BASE_SIGNAL+42

PAIR_IN_PROGRESS=BASE_SIGNAL+50
PAIR_SUCCESS=BASE_SIGNAL+51
PAIR_TIMEOUT=BASE_SIGNAL+52
PAIR_REJECT=BASE_SIGNAL+53

TEXT = {
	DONGLES_ADDED:		'Dongles added',
	NO_DONGLES: 		'No dongles to add',
	
	CYCLE_UPLOAD_DONGLE:	'Upload cycle on dongle',
	
	FILE_UPLOADED:		'File succesfully uploaded',
	FILE_FAILED:		'File failed to upload',
	
	SDP_RESOLVED:		'Service found',
	SDP_NORECORD:		'Service not provided',
	SDP_TIMEOUT:		'Failed to retrieve services',

	PAIR_IN_PROGRESS:   'Tryingto pair',
	PAIR_SUCCESS:       'Paired successfully',
	PAIR_TIMEOUT:       'User did\'t reply to pair request',
	PAIR_REJECT:        'Pairing rejected',
}

POST = {
	DONGLES_ADDED:		'uploader_dongle_added',
	NO_DONGLES: 		'uploader_no_dongle',
	
	CYCLE_UPLOAD_DONGLE:	'uploader_cycle',
	
	FILE_UPLOADED:		'uploader_success',
	FILE_FAILED:		'uploader_failed',
	
	SDP_RESOLVED:		'uploader_sdp_ok',
	SDP_NORECORD:		'uploader_sdp_no',
	SDP_TIMEOUT:		'uploader_sdp_timeout',

	PAIR_IN_PROGRESS:	'uploader_pair_start',
	PAIR_SUCCESS:		'uploader_pair_success',
	PAIR_TIMEOUT:		'uploader_pair_timeout',
	PAIR_REJECT:        'uploader_pair_reject'
}



# -*- coding: utf-8 -*-
#    OpenProximity2.0 is a proximity marketing OpenSource system.
#    Copyright (C) 2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

'''
A plugin manager.
'''

# Plugin system base
import os, re, StringIO, zipfile
import ConfigParser, pkgutil, traceback
import sys,functools

from net.aircable.utils import getLogger
logger = getLogger(__name__)

__all__=['pluginsystem']

# try getting the plugins dir from the PYTHON_PATH otherwise create a fake one
try:
    import plugins
except Exception, err:
    import new
    logger.error("no plugins dir found")
    plugins = new.module('plugins')
    plugins.__path__=[]

def find_plugin_dirs():
    '''
    Add external plugins dirs.
    '''
    return [os.path.expanduser('~/.openproximity/plugins'),
        '/usr/lib/openproximity/plugins']

# add dirs from sys.path:
plugins.__path__ = pkgutil.extend_path(plugins.__path__, 
                                              plugins.__name__)
# add dirs specific to sensorsdk:
plugins.__path__ = find_plugin_dirs() + plugins.__path__

class Plugin(object):
  '''
  A plugin instance.
  '''
  
  def __loadFromFile(self, path):
    '''
    internal method, load plugin from the file system
    '''
    p = os.path.join(path, 'plugin.py')
    if not os.path.isfile(p):
	raise Exception("plugin not suported %s" % path)
    A=file(p, 'r')
    content = A.read()
    A.close()
    content=compile(content, p, 'exec')
    glob = dict()
    loc = dict()
    eval(content, glob, loc)
    self.provides = loc
    if "post_environ" in self.provides:
	self.post_environ=self.provides['post_environ']

  def __loadFromEgg(self, path):
    '''
    internal method, load plugin from an egg file
    '''
    b=zipfile.PyZipFile(path)
    path=os.path.join(self.name, 'plugin.py')
    if not path  in b.namelist():
	raise Exception("plugin not suported %s" % path)
    A=b.open(path)
    content = A.read()
    A.close()
    content=compile(content, path, 'exec')
    glob = dict()
    loc = dict()
    eval(content, glob, loc)
    self.provides = loc
    if "post_environ" in self.provides:
	self.post_environ=self.provides['post_environ']

  def __init__(self, path=None, name=None, egg=None):
    '''
    Try to register a new plugin.
    
    @path=Path to the plugin
    @name=Name of the plugin it self
    @egg=True if the plugin is stored into an egg file
    '''
    if not egg:
      self.path = os.path.join(path, name)
    else:
      self.path = path
    if not egg:
      self.name = 'plugins.%s' % name
    else:
      self.name = name
    self.egg = egg
    if not self.egg:
      self.__loadFromFile(self.path)
    else:
      self.__loadFromEgg(self.path)
    self.enabled = self.provides.get('enabled', False)
    self.__module = None
    self.__version = None
    self.__version_info = None
    self.__rpc = None
    logger.info("Plugin: %s registered" % self.name)

  @property
  def module(self):
    '''
    The module it self
    '''
    if self.__module is None:
      self.__import_plugin()
    return self.__module

  @property
  def __version__(self):
    '''
    Version number of the plugin
    '''
    if self.__version is None:
      self.__version=getattr(self.module, '__version__', 'ND')
    return self.__version

  @property
  def __version_info__(self):
    '''
    Version info for this plugin
    '''
    if self.__version_info is None:
      self.__version_info=getattr(self.module, '__version_info__', 'ND')
    return self.__version_info

  @property
  def rpc(self):
    '''
    Information regarding RPC.
    '''
    if self.__rpc is None:
      rpc = __import__("%s.rpc" % self.name, {}, {}, ['handle', 'register'])
      self.__rpc = {}
      self.__rpc['handle'] = getattr(rpc, 'handle', None)
      self.__rpc['register'] = getattr(rpc, 'register', None)
      self.__rpc['found_action'] = getattr(rpc, 'found_action', None)
    return self.__rpc

  def __import_plugin(self):
    '''
    Try to import the plugin, if it fails then it wasn't available.
    '''
    plugin=__import__(self.name, {}, {}, [], 0)
    try:
      plugin=getattr(plugins, self.name.split('.',1)[-1])
    except Exception, err:
      logger.error('plugin was not part of plugins.*')
      logger.exception(err)
    self.__module=plugin


class PluginSystem(object):
  '''
  The plugins manager it self
  '''
  def __init__(self):
    self.plugin_infos = None

  def get_plugins(self, provides=None):
    '''
    Get plugins, if provide is defined then you can filter by provided features.
    '''
    for i in self.plugin_infos.itervalues():
      if not provides or i.provides.get(provides, None):
	yield i

  def _find_plugins_for_egg(self, egg_name):
    '''
    Find plugin in egg file
    '''
    b=zipfile.PyZipFile(egg_name)
    for a in b.namelist():
      if not a.startswith('EGG-INFO') and a.endswith('__init__.py'):
	try:
	  if not egg_name in sys.path:
	    sys.path.append(egg_name)
	  return self.load_info(egg_name, a.split('/')[0], egg=True)
	except Exception, err:
	  logger.error("Failed to load info from egg file: %s" % egg_name)
	  logger.exception(err)
    logger.info("no plugin in egg %s" % egg_name)
    
  def find_plugins(self, options):
    '''
    Init the plugin system, look for available plugins. This can only be done 
    once.
    options is a dict holding the name for each plugin and a value, if value is
    TRUE (default) then it will be enabled, otherwise plugin defaults are used
    '''
    if self.plugin_infos is not None:
      return
    logger.info("looking for plugins")
    self.plugin_infos=dict()
    for path in plugins.__path__:
      if not os.path.isdir(path):
	continue
      for entry in os.listdir(path):
	if entry.startswith('_'):
	  continue # __init__.py etc.
	if entry.endswith('.py') or os.path.isdir( os.path.join(path, entry) ):
	  try:
	    plug = self.load_info(path, entry.split('.')[0])
	  except Exception, err:
	    logger.error("Failed to load info %s" % entry)
	    continue
	if entry.endswith('.egg'):
	    plug = self._find_plugins_for_egg(os.path.join(path, entry))
	    if not plug:
	      continue

	if options.__contains__(plug.name):
	  logger.debug("found plugin in runtime settings")
	  if not options[plug.name]:
	    logger.debug("plugin disabled from settings!")
	elif not plug.enabled:
	  logger.debug("plugin disabled by default")
	  continue

	self.plugin_infos[plug.name] = plug

  def load_info(self, path, name, egg=False):
    '''
    Try to create a Plugin instance for the given plugin. If it works then 
    register the plugin in our database.
    '''
    if not egg:
      _name='plugins.%s' % name
    else:
      _name=name
    logger.info("Plugin: load_info %s" % name)
    plugin = Plugin(path, name, egg)
    logger.info("Plugin: %s ready" % _name)
    return plugin


  def post_environ(self):
    '''
    Call this method once django environ has been setup. This will tell the 
    plugins it's time to complete with the setup process.
    '''
    from django.db.models.loading import load_app
    for plugin in self.get_plugins():
      if plugin.provides.get('post_environ', False):
	plugin.module.post_environ()
    for plugin in self.get_plugins():
      load_app(plugin.name)

pluginsystem = PluginSystem()

if __name__=='__main__':
  pluginsystem.find_plugins()
  for plug in pluginsystem.get_plugins():
    print plug.provides



"""
    A simple class for creating spp servers.

    Copyright 2008 Wireless Cables Inc. <www.aircable.net>
    Copyright 2008 Naranjo, Manuel Francisco <manuel@aircable.net>

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
import socket
import dbus

from errors import *
from sppBase import *
from xml.dom.minidom import parseString

from net.aircable.utils import getLogger
logger=getLogger(__name__)

class sppServer(sppBase):
	__RecHandle = 0
	
	def __init__(self, 
		    channel=1, 
		    service='spp', 
		    device = None):
	    sppBase.__init__(self, channel, service, device);
	    logger.debug("sppServer.__init__")
	    

	def listenAndWait(self):
	    '''
		Waits for the connection, and initialziates the connections.
		
		It is recommened that you all ready have an sdp record registered
		before calling this method, and that you remove it as soon as this
		method ends.
		
		This method will block.
	    '''
	    if (self.__RecHandle <= 0):
		logger.warning("No SDP service registered");
		
	    logger.debug( 'creating socket' )
	    sock = socket.socket( socket.AF_BLUETOOTH, 
						socket.SOCK_STREAM, 
						socket.BTPROTO_RFCOMM );
	    #Let BlueZ decide outgoing port
	    logger.debug ( 'binding to %s, %i' % ( self.device , self.channel ) )
	    sock.bind( (self.device,self.channel) );

	    logger.debug ( 'Listening' )
	    sock.listen( 1 );
	    
	    logger.info ( 'Waiting for connection' )
	    (self.socket, self.peer) = sock.accept()
	    
	    logger.info( 'Connection from %s at channel %s' % 
		(self.peer[0], self.peer[1]) )
		
	    sock.close()
	    
	def __registerSDPOldAPI(self, xml):
	    logger.warning("BlueZ 3.X API doesn't allow to register records on just one specific device")
	    obj = self.bus.get_object('org.bluez', '/org/bluez')
	    self.database = dbus.Interface(obj, 'org.bluez.Database')
	    self.__RecHandle = self.database.AddRecordFromXML(xml)
	    
	def __registerSDPNewAPI(self, xml):
	    self.database = dbus.Interface(self.getAdapterObjectPath(),
				'org.bluez.Service');
	    self.__RecHandle = self.database.AddRecord(xml)
	    
	def __genXML(self, name, description):
	    return '''
<?xml version=\"1.0\" encoding=\"UTF-8\" ?>

		<record>
        <attribute id=\"0x0000\">
                <uint32 value=\"0x00010001\" />
        </attribute>
        <attribute id=\"0x0001\">
                <sequence>
                        <uuid value=\"0x1101\" />
                </sequence>
        </attribute>
        <attribute id=\"0x0004\">
                <sequence>
                        <sequence>
                                <uuid value=\"0x0100\" />
                        </sequence>
                        <sequence>
                                <uuid value=\"0x0003\" />
                                <uint8 value=\"0x%X\" />
                        </sequence>
                </sequence>
        </attribute>
        <attribute id=\"0x0005\">
                <sequence>
                        <uuid value=\"0x1002\" />
                </sequence>
        </attribute>
        <attribute id=\"0x0006\">
                <sequence>
                        <uint16 value=\"0x656e\" />
                        <uint16 value=\"0x006a\" />
                        <uint16 value=\"0x0100\" />
                </sequence>
        </attribute>
        <attribute id=\"0x0009\">
                <sequence>
                        <sequence>
                                <uuid value=\"0x1101\" />
                                <uint16 value=\"0x0100\" />
                        </sequence>
                </sequence>
        </attribute>
        <attribute id=\"0x0100\">
                <text value=\"%s\" />
        </attribute>
        <attribute id=\"0x0101\">
                <text value=\"%s\" />
        </attribute>
</record>''' % (self.channel, name, description )
	    
	
	def registerSDP(self, name="sppServer",  
				description="AIRcable SensorSDK SPP Server",
				xml_entry=None):
	    '''
		Registers the SDP record with the SDP database
		
		Arguments (all optionals):
		    name: Name for the service when you want sppServer to generate
			    xml sdp record
		    description: Description for the service when you want sppServer to generate
			    xml sdp record
		    xml_entry: If you are not going to use spp then you need to provide
			    this argument
	    '''
	    logger.info("Registering service %s named %s with description %s" %
		    (self.service, name, description) )
		    
	    if self.service != 'spp' and not xml_entry:
		raise SPPNotImplemented, "If you are not going to use spp then you need to provide the xml record"
	
	    if self.service == 'spp':
		xml = self.__genXML(name, description)
	    else:
		xml = xml_entry
		
	    logger.debug( "Registering:%s" % xml )
	    
	    if self.new_bluez_api :
		self.__registerSDPNewAPI(xml)
	    else:
		self.__registerSDPOldAPI(xml)
	    
	    logger.info( "Registered, RecHandle = 0x%X" % self.__RecHandle )
	    
	def unregisterSDP(self):
	    if self.__RecHandle < 0:
		raise SPPException, "Can't unregister SDP record if it wasn't registered first"
	    
	    logger.info("Removing SDP record handle: 0x%X" % self.__RecHandle)
	    self.database.RemoveRecord(self.__RecHandle)


if __name__ == '__main__':
    import sys
    
    if ( '--help' in sys.argv ):
	print "Usage %s [<service> <channel> <adapter>]" % (sys.argv[0])
	sys.exit(1)
    
    if ( len(sys.argv) > 1):
	serv = sys.argv[1]
    else:
	serv = 'spp'
    
    if ( len(sys.argv) > 2 ):
	chan = sys.argv[2]
    else:
	chan = 1
    
    if ( len(sys.argv) > 3 ):
	dev = sys.argv[3]
    else:
	dev = None
	
    a = sppServer(
	    channel	=chan,
	    service	=serv,
	    device	=dev
	);
    
    a.registerSDP()
    a.listenAndWait()
    a.unregisterSDP()
    
    print a.readLine()
    a.sendLine('How are you?')
    print a.readLine()
    
    a.disconnect()



from setuptools import setup, find_packages
from net.aircable import __version__

setup(
    name='aircable-library-op',
    version=__version__,
    description='AIRcable libraries for BlueZ Development',
    author='Manuel Francisco Naranjo',
    author_email='manuel@aircable.net',
    url='https://github.com/OpenProximity/aircable-library',
    packages=find_packages(),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
    include_package_data=True,
    zip_safe=False,
    install_requires = [
        #'dbus', 
        #'gobject', 
        'simplejson']
)



#    OpenProximity2.0 is a proximity marketing OpenSource system.
#    Copyright (C) 2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
from django.db import models
from pickle import loads, dumps

'''
Some helper django db fields used by OpenProximity and it's plugins.
'''

class PickledField(models.CharField):
    '''
	A simple class that can be used to store settings in django db
    '''
    __metaclass__ = models.SubfieldBase
    
    def to_python(self, value):
	if type(value) is unicode:
	    value=str(value)
	try:
	    return loads(value)
	except:
	    return value
	    
    def get_db_prep_value(self, value):
	if type(value) is unicode:
	    value=str(value)
	try:
	    if type(value) is str:
		value=eval(value)
	    if type(value) is not str:
		value = dumps(value)
	except:
	    pass
	return ''.join(value)

from south.modelsinspector import add_introspection_rules
add_introspection_rules([], ["^net\.aircable\.fields\.PickledField"])



# Written by Naranjo Manuel Francisco <manuel@aircable.net>

"""Worker queues for Python (also called thread pools).

This class queues work and distributes it over a set of slave process. The
slaves are Callable instances that get called in their own process. 

The enqueue() function is used to queue function calls.

"""
import multiprocessing, functools
from multiprocessing import Lock, Semaphore
from net.aircable.utils import getLogger
logger=getLogger(__name__)

__all__=['Manager', 'Lock', 'Semaphore']

class Worker(multiprocessing.Process):
    def __init__(self, id, manager, queue, out, semaphore, *args, **kwargs):
	multiprocessing.Process.__init__(self,*args,**kwargs)
	self.id = id
	self.manager = manager
	self.queue = queue
	self.running = False
	self.out=out
	self.semaphore=semaphore

    def run(self):
	# takes an element from the queue and process it
	while [ 1 ]:
	    func, args, kwargs = self.queue.get(True)
	    self.running = True
	    logger.debug("got func from queue")

	    # make the call the manager has all ready make sure this
	    # call is a partial
	    kwargs['out']=self.out
	    kwargs['semaphore']=self.semaphore
	    func(*args,**kwargs)

	    # allow others to know we're free again
	    self.running = False

    def busy(self):
	return self.running

class Manager(object):
    '''A class that manages subprocess calls'''

    def __init__(self, pids=7):
	self.queue = multiprocessing.Queue()
	self.out = multiprocessing.Queue()
	self.semaphore = multiprocessing.Semaphore()
	self.workers = [ Worker( b, self, self.queue, self.out, self.semaphore ) for b in range(0,pids) ]

    def put(self, function, *args, **kwargs):
#	part = functools.partial(function, *args, **kwargs)
	self.queue.put([function, args, kwargs])
	
    def put_out(self, *args, **kwargs):
	self.out.put([args,kwargs])
	
    def get_out(self):
	return self.out.get(True)

    def running_count(self):
	i = 0
	for j in self.workers:
	    if j.busy(): i+=1
	return i
	
    def start(self):
	for j in self.workers:
	    j.start()

    def stop(self):
	self.queue.close()
	self.queue.join_thread()
	self.out.close()
	self.out.join_thread()
	for j in self.workers:
	    j.terminate()

if __name__=='__main__':
    def do_work(val):
	print val
	
    import time
    from random import randint
    # tester
    b=Manager(10)
    print "manager created"

    print "added to queue"
    b.start()
    
    for k in range(0,100):
	b.put(do_work, k)
	time.sleep(randint(0,200)*.001)
    
    print "stopping"
    b.stop()
